/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <inttypes.h>
#include <nrfx_gpiote.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>
#include <zephyr/sys/util.h>
#include <zephyr/sys/printk.h>
#include "button.h"
#include "error.h"
#include "led.h"
#include "localtime.h"
#include "nrf52832_board.h"
#include "pwr_mgmt.h"

#define BUTTON_TIMER_MSEC 10000

/*
 * Get configuration from the devicetree alias. This is mandatory.
 */
#define BOOT_BUTTON_NODE DT_ALIAS(sw0)

#if !DT_NODE_HAS_STATUS(BOOT_BUTTON_NODE, okay)
#error "Unsupported board: mcuboot-button0 devicetree alias is not defined"
#endif

LOG_MODULE_REGISTER(button);

/*
 * Helper macro for initializing a gpio_dt_spec from the devicetree
 * with fallback values when the nodes are missing.
 */
static const struct gpio_dt_spec _gButton   = GPIO_DT_SPEC_GET_OR(BOOT_BUTTON_NODE, gpios, {0});

//static struct gpio_callback button_cb_data;

static void _buttonHandler(struct k_timer *dummy) {
    printk("%" PRId64 "\n", k_uptime_get());
    ltPrintCurrentDatetime();
    int val = gpio_pin_get_dt(&_gButton);
    printk("val=%d, BUTTON_PRESS=%d\n", val, BUTTON_PRESS);
    if (val == BUTTON_PRESS) {
        const power_state_t state = pwrMgmtGetPowerState();
        if (state == POWER_STATE_DOWN) {
            printk("power up\n");
            pwrMgmtPowerUp();
            ledSetG();
        } else {
            printk("power down\n");
            pwrMgmtPowerDown();
        }
    }
}

K_TIMER_DEFINE(_gButtonTimer, _buttonHandler, NULL);

static void _buttonPressed(nrfx_gpiote_pin_t     pin,
                          nrfx_gpiote_trigger_t trigger,
                          void *                p_context) {
//void button_pressed(const struct device *dev, struct gpio_callback *cb, uint32_t pins) {
    printk("Button pressed at %" PRIu32 "\n", k_cycle_get_32());
    printk("start a timer\n");
    ltPrintCurrentDatetime();
    printk("%" PRId64 "\n", k_uptime_get());
    k_timer_start(&_gButtonTimer, K_MSEC(BUTTON_TIMER_MSEC), K_NO_WAIT);
}

int buttonInit(void) {
    //int ret;
    nrfx_err_t errCode;

    //ledInit();

    if (!device_is_ready(_gButton.port)) {
        printk("Error: button device %s is not ready\n", _gButton.port->name);
        return ERROR_BUTTON_INITIALIZE_FAIL;
    }

    printk("device_is_ready() ok\n");
#if 0
    nrfx_gpiote_in_config_t inConfig = NRFX_GPIOTE_CONFIG_IN_SENSE_TOGGLE(true);
    inConfig.sense = NRF_GPIOTE_POLARITY_LOTOHI;
    inConfig.pull  = NRF_GPIO_PIN_PULLDOWN; // interrpt: raising edge
    errCode = nrfx_gpiote_in_init(BUTTON_1, &inConfig, button_pressed);
    
    int val = gpio_pin_get_dt(&_gButton);
    printk("button state=%d\n", val);
    //if (val == BUTTON_PRESS) 
    {
        printk("start a timer\n");
        ltPrintCurrentDatetime();
        printk("%" PRId64 "\n", k_uptime_get());
        k_timer_start(&_gButtonTimer, K_MSEC(BUTTON_TIMER_MSEC), K_NO_WAIT);
    }
#else
	nrfx_gpiote_input_config_t inputConfig = {.pull = NRF_GPIO_PIN_PULLUP};
	
	uint8_t gpioteChannel;
	errCode = nrfx_gpiote_channel_alloc(&gpioteChannel);
	nrfx_gpiote_trigger_config_t triggerConfig = {
		.trigger 		= NRFX_GPIOTE_TRIGGER_TOGGLE, 
		.p_in_channel 	= &gpioteChannel
	};

	nrfx_gpiote_handler_config_t handleConfig = {
		.handler 	= &_buttonPressed,
		.p_context 	= NULL
	};
	
	errCode = nrfx_gpiote_input_configure(_gButton.pin, &inputConfig, &triggerConfig, &handleConfig);
	if (errCode != NRFX_SUCCESS) {
		printk("Error: nrfx_gpiote_input_configure() failed for button, errCode=0x%x\n", errCode);
        return ERROR_BUTTON_INITIALIZE_FAIL;
	}
	
	nrfx_gpiote_trigger_enable(_gButton.pin, true);

    printk("start a timer\n");
    ltPrintCurrentDatetime();
    printk("%" PRId64 "\n", k_uptime_get());
    k_timer_start(&_gButtonTimer, K_MSEC(BUTTON_TIMER_MSEC), K_NO_WAIT);
#endif	
    return 0;
}
